@@ -26,6 +26,24 @@
 #include "xklavier_private.h"
 #include "xklavier_private_xkb.h"
 
+#ifdef HAVE_XINPUT
+#include "config.h"
+#include "X11/extensions/XInput.h"
+
+static gint
+xkl_xinput_process_x_event(XklEngine * engine, XEvent * xev)
+{
+	XDevicePresenceNotifyEvent *dpne =
+	    (XDevicePresenceNotifyEvent *) xev;
+	xkl_debug(200, "XInput event detected: %d\n", dpne->devchange);
+	if (dpne->devchange == DeviceEnabled) {
+		xkl_debug(150, "Device enabled: %d\n", dpne->deviceid);
+		g_signal_emit_by_name(engine, "X-new-device");
+	}
+	return 1;
+}
+#endif
+
 /*
  * XKB event handler
  */
@@ -38,11 +56,19 @@ xkl_xkb_process_x_event(XklEngine * engine, XEvent * xev)
 	guint inds;
 	XkbEvent *kev = (XkbEvent *) xev;
 
-	if (xev->type != xkl_engine_backend(engine, XklXkb, event_type))
+	if (!
+	    (xkl_engine_is_listening_for(engine, XKLL_MANAGE_WINDOW_STATES)
+	     | xkl_engine_is_listening_for(engine,
+					   XKLL_TRACK_KEYBOARD_STATE)))
 		return 0;
 
-	if (!(xkl_engine_priv(engine, listener_type) &
-	      (XKLL_MANAGE_WINDOW_STATES | XKLL_TRACK_KEYBOARD_STATE)))
+#ifdef HAVE_XINPUT
+	/* Special case XInput event */
+	if (xev->type == xkl_engine_backend(engine, XklXkb, xi_event_type))
+		return xkl_xinput_process_x_event(engine, xev);
+#endif
+
+	if (xev->type != xkl_engine_backend(engine, XklXkb, event_type))
 		return 0;
 
 	xkl_debug(150, "Xkb event detected\n");
@@ -63,8 +89,8 @@ xkl_xkb_process_x_event(XklEngine * engine, XEvent * xev)
 		if (kev->state.changed & GROUP_CHANGE_MASK)
 			xkl_engine_process_state_modification(engine,
 							      GROUP_CHANGED,
-							      kev->state.
-							      locked_group,
+							      kev->
+							      state.locked_group,
 							      0, FALSE);
 		else {		/* ...not interested... */
 
@@ -75,8 +101,8 @@ xkl_xkb_process_x_event(XklEngine * engine, XEvent * xev)
 				xkl_debug(0,
 					  "ATTENTION! Currently cached group %d is not equal to the current group from the event: %d\n!",
 					  xkl_engine_priv(engine,
-							  curr_state).
-					  group, kev->state.locked_group);
+							  curr_state).group,
+					  kev->state.locked_group);
 		}
 
 		break;
@@ -138,6 +164,26 @@ xkl_xkb_process_x_event(XklEngine * engine, XEvent * xev)
 #endif
 }
 
+/*
+ * XKB error handler
+ */
+gint
+xkl_xkb_process_x_error(XklEngine * engine, XErrorEvent * xerev)
+{
+#ifdef HAVE_XINPUT
+	/* Ignore XInput errors */
+	if (xerev->error_code >=
+	    xkl_engine_backend(engine, XklXkb, xi_error_code)
+	    && xerev->error_code <=
+	    (xkl_engine_backend(engine, XklXkb, xi_error_code) +
+	     XI_BadClass))
+
+		return 1;
+#endif
+
+	return 0;
+}
+
 void
 xkl_xkb_set_indicators(XklEngine * engine, const XklState * window_state)
 {
@@ -150,8 +196,8 @@ xkl_xkb_set_indicators(XklEngine * engine, const XklState * window_state)
 	ForPhysIndicators(i, bit) if (cached->names->indicators[i] != None) {
 		gboolean status;
 		status = xkl_xkb_set_indicator(engine, i,
-					       (window_state->
-						indicators & bit) != 0);
+					       (window_state->indicators &
+						bit) != 0);
 		xkl_debug(150, "Set indicator \"%s\"/%d to %d: %d\n",
 			  xkl_engine_backend(engine, XklXkb,
 					     indicator_names)[i],
